@@ -22,6 +22,7 @@ import shutil
 import subprocess
 import sys
 import tarfile
+import time
 
 import openstack
 
@@ -32,7 +33,8 @@ from build_utils import (
     calc_options_hash,
     subprocess_with_log,
     subprocess_with_capture,
-    upload_file
+    upload_file,
+    is_macos
 )
 import build_utils
 
@@ -91,7 +93,7 @@ def main():
     # Differentiate between macos versions: it's possible to have the same label
     # for different macos versions, especially different minor versions.
     macos_version_prefix = ''
-    if 'Darwin' == platform.system():
+    if is_macos():
         macos_version_tuple = platform.mac_ver()
         macos_version = macos_version_tuple[0]
         macos_version_prefix = f'{macos_version}/'
@@ -253,10 +255,9 @@ def cleanup_previous_build():
 def git_pull(directory: str, repository: str, branch: str):
     returncode = 1
 
-    for _ in range(5):
-        if returncode == 0:
-            break
-
+    max_attempts = 6
+    sleep_time_unit = 3
+    for attempt in range(1, max_attempts+1):
         targetdir = os.path.join(WORKDIR, directory)
         if os.path.exists(os.path.join(targetdir, ".git")):
             returncode = subprocess_with_log(f"""
@@ -269,6 +270,13 @@ def git_pull(directory: str, repository: str, branch: str):
             returncode = subprocess_with_log(f"""
                 git clone --branch {branch} --single-branch {repository} "{targetdir}"
             """)
+        
+        if returncode == 0:
+            return
+
+        sleep_time = sleep_time_unit * attempt
+        build_utils.print_warning(f"""Attempt {attempt}: failed to pull/clone branch. Retrying in {sleep_time} seconds...""")
+        time.sleep(sleep_time)
 
     if returncode != 0:
         die(returncode, f"Failed to pull {branch}")
@@ -282,7 +290,12 @@ def download_artifacts(obj_prefix: str):
         print(f"\nExtracting archive {tar_path}")
 
         with tarfile.open(tar_path) as tar:
-            tar.extractall(WORKDIR)
+            # TODO: Simplify after ROOT is Python 3.12+ only
+            # c.f. https://docs.python.org/3.12/library/tarfile.html#extraction-filters
+            if hasattr(tarfile, "data_filter"):
+                tar.extractall(WORKDIR, filter="data")
+            else:
+                tar.extractall(WORKDIR)
 
         build_utils.log.add(f'\ncd {WORKDIR} && tar -xf {tar_path}\n')
 
@@ -528,7 +541,7 @@ def relatedrepo_GetClosestMatch(repo_name: str, origin: str, upstream: str):
     if current_head == "latest-stable":
       # Resolve the 'latest-stable' branch to the latest merged head/tag
       current_head = get_stdout_subprocess(f"""
-           git --git-dir={gitdir} for-each-ref --points-at=latest-stable^2 --format=%\(refname:short\))
+           git --git-dir={gitdir} for-each-ref --points-at=latest-stable^2 --format=%\\(refname:short\\)
            """, "Failed capture of lastest-stable underlying branch name")
       return fetch_url, current_head
 
