@@ -64,7 +64,7 @@ TBits& TBits::operator=(const TBits& rhs)
          fAllBits = new UChar_t[fNbytes];
          memcpy(fAllBits,rhs.fAllBits,fNbytes);
       } else {
-         fAllBits = 0;
+         fAllBits = nullptr;
       }
    }
    return *this;
@@ -84,7 +84,7 @@ TBits::~TBits()
 void TBits::Clear(Option_t * /*option*/)
 {
    delete [] fAllBits;
-   fAllBits = 0;
+   fAllBits = nullptr;
    fNbits   = 0;
    fNbytes  = 0;
 }
