@@ -30,15 +30,14 @@
 
 **************************************************************************/
 
-//////////////////////////////////////////////////////////////////////////
-//                                                                      //
-// TGMdiDecorFrame, TGMdiTitleBar, TGMdiButtons, TGMdiTitleIcon,        //
-// TGMdiWinResizer, TGMdiVerticalWinResizer, TGMdiHorizontalWinResizer, //
-// and TGMdiCornerWinResizer.                                           //
-//                                                                      //
-// This file contains all different MDI frame decoration classes.       //
-//                                                                      //
-//////////////////////////////////////////////////////////////////////////
+
+/** \class TGMdiDecorFrame
+    \ingroup guiwidgets
+
+This file contains all different MDI frame decoration classes.
+
+*/
+
 
 #include <cstdio>
 #include <cstdlib>
@@ -548,7 +547,7 @@ Bool_t TGMdiTitleBar::HandleButton(Event_t *event)
 ////////////////////////////////////////////////////////////////////////////////
 /// Process messages for title bar.
 
-Bool_t TGMdiTitleBar::ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2)
+Bool_t TGMdiTitleBar::ProcessMessage(Longptr_t msg, Longptr_t parm1, Longptr_t parm2)
 {
    switch (GET_MSG(msg)) {
       case kC_COMMAND:
