@@ -35,6 +35,7 @@
 #include "TArrayL64.h"
 #include "TArrayF.h"
 #include "TArrayD.h"
+#include "TDirectory.h"
 #include "Foption.h"
 
 #include "TVectorFfwd.h"
@@ -44,6 +45,10 @@
 
 #include <cfloat>
 #include <string>
+#include <stdexcept>
+#include <type_traits>
+#include <array>
+#include <numeric>
 
 class TF1;
 class TH1D;
@@ -55,6 +60,50 @@ class TVirtualFFT;
 class TVirtualHistPainter;
 class TRandom;
 
+namespace ROOT::Internal {
+/**
+ * \brief Creates a sliced copy of the given histogram.
+ *
+ * \tparam T The type of the histogram.
+ * \param histo The histogram to slice.
+ * \param args A vector of integers specifying the low and upper edges of the slice for each dimension.
+ *
+ * \return A new histogram object that is a sliced version of the input histogram.
+ */
+template <typename T>
+T Slice(const T &histo, std::vector<Int_t> &args)
+{
+   static_assert(std::is_pointer_v<decltype(histo.fArray)>,
+                 "The type of the histogram to slice must expose the internal array data.");
+
+   TDirectory::TContext _{nullptr};
+   T slicedHisto = histo;
+
+   using ValueType = std::remove_pointer_t<decltype(slicedHisto.fArray)>;
+   slicedHisto.template SliceHistoInPlace<ValueType>(args, slicedHisto.fArray, slicedHisto.fN);
+   return slicedHisto;
+}
+
+/**
+ * \brief Sets the content of a slice in a histogram.
+ *
+ * \tparam T The type of the histogram.
+ * \param histo The histogram to set the slice content for.
+ * \param input A vector of values to assign to the bins in the specified slice. All input types are converted
+ *              to Double_t as that's the type `SetBinContent` eventually uses.
+ * \param sliceEdges A vector of pairs specifying the low and upper edges of the slice for each dimension.
+ */
+template <typename T>
+void SetSliceContent(T &histo, const std::vector<Double_t> &input,
+                     const std::vector<std::pair<Int_t, Int_t>> &sliceEdges)
+{
+   static_assert(std::is_pointer_v<decltype(histo.fArray)>,
+                 "The type of the histogram to slice must expose the internal array data.");
+
+   using ValueType = std::remove_pointer_t<decltype(histo.fArray)>;
+   histo.template SetSliceContent<ValueType>(input, sliceEdges, histo.fArray);
+}
+} // namespace ROOT::Internal
 
 class TH1 : public TNamed, public TAttLine, public TAttFill, public TAttMarker {
 
@@ -83,6 +132,17 @@ public:
          kNeutral = 2,  ///< Adapt to the global flag
    };
 
+   /// Enumeration specifying inconsistencies between two histograms,
+   /// in increasing severity.
+   enum  EInconsistencyBits {
+         kFullyConsistent = 0,
+         kDifferentLabels = BIT(0),
+         kDifferentBinLimits = BIT(1),
+         kDifferentAxisLimits = BIT(2),
+         kDifferentNumberOfBins = BIT(3),
+         kDifferentDimensions = BIT(4)
+   };
+
    friend class TH1Merger;
 
 protected:
@@ -126,6 +186,186 @@ private:
    TH1(const TH1&) = delete;
    TH1& operator=(const TH1&) = delete;
 
+   /**
+    * \brief Slices a histogram in place based on the specified bin ranges for each dimension.
+    *
+    * This function modifies the histogram by extracting a sub-region defined by the provided
+    * bin ranges for each dimension. The resulting histogram will have updated bin counts,
+    * edges, and contents. Bin contents outside the range fall into the flow bins.
+    * The histogram's internal data array is freed and reallocated by this function.
+    * This function is used by the python implementation of the Unified Histogram Interface (UHI)
+    * for slicing.
+    *
+    * \tparam ValueType The type of the histogram's data.
+    * \param args A vector of integers specifying the low and upper edges of the slice for each dimension.
+    * \param dataArray A pointer to the histogram's data array.
+    * \param fN The size of dataArray.
+    */
+   template <typename ValueType>
+   void SliceHistoInPlace(std::vector<Int_t> &args, ValueType *&dataArray, Int_t &fN)
+   {
+      constexpr Int_t kMaxDim = 3;
+      Int_t ndim = args.size() / 2;
+      if (ndim != fDimension) {
+         throw std::invalid_argument(
+            Form("Number of dimensions in slice (%d) does not match histogram dimension (%d).", ndim, fDimension));
+      }
+
+      // Compute new bin counts and edges
+      std::array<Int_t, kMaxDim> nBins{}, totalBins{};
+      std::array<Double_t, kMaxDim> lowEdge{}, upEdge{};
+      for (decltype(ndim) d = 0; d < ndim; ++d) {
+         const auto &axis = (d == 0 ? fXaxis : d == 1 ? fYaxis : fZaxis);
+         auto start = std::max(1, args[d * 2]);
+         auto end = std::min(axis.GetNbins() + 1, args[d * 2 + 1]);
+         nBins[d] = end - start;
+         lowEdge[d] = axis.GetBinLowEdge(start);
+         upEdge[d] = axis.GetBinLowEdge(end);
+         totalBins[d] = axis.GetNbins() + 2;
+         args[2 * d] = start;
+         args[2 * d + 1] = end;
+      }
+
+      // Compute layout sizes for slice
+      size_t rowSz = nBins[0] + 2;
+      size_t planeSz = rowSz * (ndim > 1 ? nBins[1] + 2 : 1);
+      size_t newSize = planeSz * (ndim > 2 ? nBins[2] + 2 : 1);
+
+      // Allocate the new array
+      auto *newArr = new ValueType[newSize]();
+
+      auto rowIncr = 1 + (ndim > 1 ? (rowSz - 1) : 0);
+      ValueType under = 0, over = 0;
+      Bool_t firstUnder = false;
+      Int_t lastOverIdx = 0;
+
+      // Copy the valid slice bins
+      size_t dstIdx = 1 + (ndim > 1 ? rowSz : 0) + (ndim > 2 ? planeSz : 0);
+      for (auto z = (ndim > 2 ? args[4] : 0); z < (ndim > 2 ? args[5] : 1); ++z) {
+         for (auto y = (ndim > 1 ? args[2] : 0); y < (ndim > 1 ? args[3] : 1); ++y) {
+            size_t rowStart = z * totalBins[1] * totalBins[0] + y * totalBins[0] + args[0];
+            std::copy_n(dataArray + rowStart, nBins[0], newArr + dstIdx);
+            if (!firstUnder) {
+               under = std::accumulate(dataArray, dataArray + rowStart, ValueType{});
+               firstUnder = true;
+            } else {
+               under += std::accumulate(dataArray + rowStart - args[0], dataArray + rowStart, ValueType{});
+            }
+            lastOverIdx = rowStart - args[0] + totalBins[0];
+            over += std::accumulate(dataArray + rowStart + nBins[0], dataArray + lastOverIdx, ValueType{});
+            dstIdx += rowIncr;
+         }
+         if (ndim > 2) {
+            dstIdx += 2 * rowIncr;
+         }
+      }
+
+      // Copy the flow bins
+      over += std::accumulate(dataArray + lastOverIdx, dataArray + fN, ValueType{});
+      newArr[0] = under;
+      newArr[newSize - 1] = over;
+
+      // Assign the new array
+      delete[] dataArray;
+      dataArray = newArr;
+
+      // Reconfigure Axes
+      if (ndim == 1) {
+         this->SetBins(nBins[0], lowEdge[0], upEdge[0]);
+      } else if (ndim == 2) {
+         this->SetBins(nBins[0], lowEdge[0], upEdge[0], nBins[1], lowEdge[1], upEdge[1]);
+      } else if (ndim == 3) {
+         this->SetBins(nBins[0], lowEdge[0], upEdge[0], nBins[1], lowEdge[1], upEdge[1], nBins[2], lowEdge[2],
+                       upEdge[2]);
+      }
+
+      // Update the statistics
+      ResetStats();
+      fEntries = std::accumulate(dataArray, dataArray + newSize, 0.0);
+   }
+
+   template <typename T>
+   friend T ROOT::Internal::Slice(const T &histo, std::vector<Int_t> &args);
+
+   /**
+    * \brief Sets the content of a slice of bins in a histogram.
+    *
+    * This function allows setting the content of a slice of bins in a histogram
+    * by specifying the edges of the slice and the corresponding values to assign.
+    *
+    * \tparam ValueType The type of the histogram's data.
+    * \param values A vector of values to assign to the bins in the specified slice.
+    * \param sliceEdges A vector of pairs specifying the low and upper edges of the slice for each dimension.
+    * \param dataArray A pointer to the histogram's data array.
+    */
+   template <typename ValueType>
+   void SetSliceContent(const std::vector<Double_t> &values, const std::vector<std::pair<Int_t, Int_t>> &sliceEdges,
+                        ValueType *dataArray)
+   {
+      const Int_t ndim = sliceEdges.size();
+      if (ndim != fDimension) {
+         throw std::invalid_argument(Form(
+            "Number of edges in the specified slice (%d) does not match histogram dimension (%d).", ndim, fDimension));
+      }
+
+      // Get the indices to set
+      auto getSliceIndices = [](const std::vector<std::pair<Int_t, Int_t>> &edges) -> std::vector<std::vector<Int_t>> {
+         const auto dim = edges.size();
+         if (dim == 0) {
+            return {};
+         }
+
+         std::vector<std::vector<Int_t>> slices(dim);
+         for (size_t d = 0; d < dim; ++d) {
+            for (auto val = edges[d].first; val < edges[d].second; ++val) {
+               slices[d].push_back(val);
+            }
+         }
+
+         size_t totalCombinations = 1;
+         for (const auto &slice : slices) {
+            totalCombinations *= slice.size();
+         }
+
+         std::vector<std::vector<Int_t>> result(totalCombinations, std::vector<Int_t>(3, 0));
+         for (size_t d = 0; d < slices.size(); ++d) {
+            size_t repeat = 1;
+            for (size_t i = d + 1; i < slices.size(); ++i) {
+               repeat *= slices[i].size();
+            }
+
+            size_t index = 0;
+            for (size_t i = 0; i < totalCombinations; ++i) {
+               result[i][d] = slices[d][(index / repeat) % slices[d].size()];
+               ++index;
+            }
+         }
+
+         return result;
+      };
+
+      auto sliceIndices = getSliceIndices(sliceEdges);
+
+      if (values.size() != sliceIndices.size()) {
+         throw std::invalid_argument("Number of provided values does not match number of bins to set.");
+      }
+
+      for (size_t i = 0; i < sliceIndices.size(); ++i) {
+         auto globalBin = this->GetBin(sliceIndices[i][0], sliceIndices[i][1], sliceIndices[i][2]);
+
+         // Set the bin content
+         dataArray[globalBin] = values[i];
+      }
+
+      // Update the statistics
+      ResetStats();
+      fEntries += std::accumulate(values.begin(), values.end(), 0.0);
+   }
+
+   template <typename T>
+   friend void ROOT::Internal::SetSliceContent(T &histo, const std::vector<Double_t> &input,
+                                               const std::vector<std::pair<Int_t, Int_t>> &sliceEdges);
+
 protected:
    TH1();
    TH1(const char *name,const char *title,Int_t nbinsx,Double_t xlow,Double_t xup);
@@ -135,6 +375,7 @@ protected:
    Int_t            AxisChoice(Option_t *axis) const;
    virtual Int_t    BufferFill(Double_t x, Double_t w);
    virtual Bool_t   FindNewAxisLimits(const TAxis* axis, const Double_t point, Double_t& newMin, Double_t &newMax);
+   TString          ProvideSaveName(Option_t *option, Bool_t testfdir = kFALSE);
    virtual void     SavePrimitiveHelp(std::ostream &out, const char *hname, Option_t *option = "");
    static Bool_t    RecomputeAxisLimits(TAxis& destAxis, const TAxis& anAxis);
    static Bool_t    SameLimitsAndNBins(const TAxis& axis1, const TAxis& axis2);
@@ -156,7 +397,6 @@ protected:
    static bool CheckBinLabels(const TAxis* a1, const TAxis* a2);
    static bool CheckEqualAxes(const TAxis* a1, const TAxis* a2);
    static bool CheckConsistentSubAxes(const TAxis *a1, Int_t firstBin1, Int_t lastBin1, const TAxis *a2, Int_t firstBin2=0, Int_t lastBin2=0);
-   static int CheckConsistency(const TH1* h1, const TH1* h2);
    int LoggedInconsistency(const char* name, const TH1* h1, const TH1* h2, bool useMerge=false) const;
 
 public:
@@ -189,9 +429,13 @@ public:
 
    virtual Bool_t   Add(TF1 *h1, Double_t c1=1, Option_t *option="");
    virtual Bool_t   Add(const TH1 *h1, Double_t c1=1);
-   virtual Bool_t   Add(const TH1 *h, const TH1 *h2, Double_t c1=1, Double_t c2=1); // *MENU*
-   virtual void     AddBinContent(Int_t bin);
-   virtual void     AddBinContent(Int_t bin, Double_t w);
+   virtual Bool_t   Add(const TH1 *h, const TH1 *h2, Double_t c1=1, Double_t c2=1);
+   /// Increment bin content by 1.
+   /// Passing an out-of-range bin leads to undefined behavior
+   virtual void     AddBinContent(Int_t bin) = 0;
+   /// Increment bin content by a weight w.
+   /// Passing an out-of-range bin leads to undefined behavior
+   virtual void     AddBinContent(Int_t bin, Double_t w) = 0;
    static  void     AddDirectory(Bool_t add=kTRUE);
    static  Bool_t   AddDirectoryStatus();
            void     Browse(TBrowser *b) override;
@@ -199,6 +443,7 @@ public:
    virtual Double_t Chi2Test(const TH1* h2, Option_t *option = "UU", Double_t *res = nullptr) const;
    virtual Double_t Chi2TestX(const TH1* h2, Double_t &chi2, Int_t &ndf, Int_t &igood,Option_t *option = "UU",  Double_t *res = nullptr) const;
    virtual Double_t Chisquare(TF1 * f1, Option_t *option = "") const;
+   static  Int_t    CheckConsistency(const TH1* h1, const TH1* h2);
    virtual void     ClearUnderflowAndOverflow();
    virtual Double_t ComputeIntegral(Bool_t onlyPositive = false);
            TObject* Clone(const char *newname = "") const override;
@@ -207,7 +452,7 @@ public:
            Int_t    DistancetoPrimitive(Int_t px, Int_t py) override;
    virtual Bool_t   Divide(TF1 *f1, Double_t c1=1);
    virtual Bool_t   Divide(const TH1 *h1);
-   virtual Bool_t   Divide(const TH1 *h1, const TH1 *h2, Double_t c1=1, Double_t c2=1, Option_t *option=""); // *MENU*
+   virtual Bool_t   Divide(const TH1 *h1, const TH1 *h2, Double_t c1=1, Double_t c2=1, Option_t *option="");
            void     Draw(Option_t *option = "") override;
    virtual TH1     *DrawCopy(Option_t *option="", const char * name_postfix = "_copy") const;
    virtual TH1     *DrawNormalized(Option_t *option="", Double_t norm=1) const;
@@ -222,7 +467,8 @@ public:
    virtual Int_t    Fill(const char *name, Double_t w);
    virtual void     FillN(Int_t ntimes, const Double_t *x, const Double_t *w, Int_t stride=1);
    virtual void     FillN(Int_t, const Double_t *, const Double_t *, const Double_t *, Int_t) {}
-   virtual void     FillRandom(const char *fname, Int_t ntimes=5000, TRandom * rng = nullptr);
+   virtual void     FillRandom(TF1 *f1, Int_t ntimes=5000, TRandom * rng = nullptr);
+           void     FillRandom(const char *fname, Int_t ntimes=5000, TRandom * rng = nullptr);
    virtual void     FillRandom(TH1 *h, Int_t ntimes=5000, TRandom * rng = nullptr);
    virtual Int_t    FindBin(Double_t x, Double_t y=0, Double_t z=0);
    virtual Int_t    FindFixBin(Double_t x, Double_t y=0, Double_t z=0) const;
@@ -309,7 +555,10 @@ public:
    virtual void     GetStats(Double_t *stats) const;
    virtual Double_t GetStdDev(Int_t axis=1) const;
    virtual Double_t GetStdDevError(Int_t axis=1) const;
-   virtual Double_t GetSumOfWeights() const;
+   Double_t         GetSumOfAllWeights(const bool includeOverflow) const;
+   /// Return the sum of weights across all bins excluding under/overflows.
+   /// \see TH1::GetSumOfAllWeights()
+   virtual Double_t GetSumOfWeights() const { return GetSumOfAllWeights(false); }
    virtual TArrayD *GetSumw2() {return &fSumw2;}
    virtual const TArrayD *GetSumw2() const {return &fSumw2;}
    virtual Int_t    GetSumw2N() const {return fSumw2.fN;}
@@ -346,7 +595,7 @@ public:
            Long64_t Merge(TCollection *list, Option_t * option);
    virtual Bool_t   Multiply(TF1 *f1, Double_t c1=1);
    virtual Bool_t   Multiply(const TH1 *h1);
-   virtual Bool_t   Multiply(const TH1 *h1, const TH1 *h2, Double_t c1=1, Double_t c2=1, Option_t *option=""); // *MENU*
+   virtual Bool_t   Multiply(const TH1 *h1, const TH1 *h2, Double_t c1=1, Double_t c2=1, Option_t *option="");
            void     Paint(Option_t *option = "") override;
            void     Print(Option_t *option = "") const override;
    virtual void     PutStats(Double_t *stats);
@@ -358,7 +607,7 @@ public:
    virtual void     ResetStats();
            void     SaveAs(const char *filename = "hist", Option_t *option = "") const override;  // *MENU*
            void     SavePrimitive(std::ostream &out, Option_t *option = "") override;
-   virtual void     Scale(Double_t c1=1, Option_t *option="");
+   virtual void     Scale(Double_t c1=1, Option_t *option="");  // *MENU*
    virtual void     SetAxisColor(Color_t color=1, Option_t *axis="X");
    virtual void     SetAxisRange(Double_t xmin, Double_t xmax, Option_t *axis="X");
    virtual void     SetBarOffset(Float_t offset=0.25) {fBarOffset = Short_t(1000*offset);}
@@ -428,6 +677,7 @@ public:
            void     UseCurrentStyle() override;
    static  TH1     *TransformHisto(TVirtualFFT *fft, TH1* h_output,  Option_t *option);
 
+   static void      SavePrimitiveFunctions(std::ostream &out, const char *varname, TList *lst);
 
    // TODO: Remove obsolete methods in v6-04
    virtual Double_t GetCellContent(Int_t binx, Int_t biny) const
@@ -444,8 +694,15 @@ public:
    ClassDefOverride(TH1,8)  //1-Dim histogram base class
 
 protected:
-   virtual Double_t RetrieveBinContent(Int_t bin) const;
-   virtual void     UpdateBinContent(Int_t bin, Double_t content);
+
+   /// Raw retrieval of bin content on internal data structure
+   /// see convention for numbering bins in TH1::GetBin
+   virtual Double_t RetrieveBinContent(Int_t bin) const = 0;
+
+   /// Raw update of bin content on internal data structure
+   /// see convention for numbering bins in TH1::GetBin
+   virtual void     UpdateBinContent(Int_t bin, Double_t content) = 0;
+
    virtual Double_t GetBinErrorSqUnchecked(Int_t bin) const { return fSumw2.fN ? fSumw2.fArray[bin] : RetrieveBinContent(bin); }
 };
 
