@@ -96,9 +96,8 @@ for single nodes.
 #include <fstream>
 #include <sstream>
 
-using std::cout, std::endl, std::ostream, std::string, std::set, std::map, std::istream, std::pair, std::ofstream, std::make_pair;
+using std::ostream, std::string, std::set, std::map, std::istream, std::pair, std::ofstream, std::make_pair;
 
-ClassImp(RooAbsArg);
 
 bool RooAbsArg::_verboseDirty(false) ;
 bool RooAbsArg::_inhibitDirty(false) ;
@@ -180,12 +179,12 @@ RooAbsArg::~RooAbsArg()
     if (_verboseDirty) {
 
       if (first) {
-   cxcoutD(Tracing) << "RooAbsArg::dtor(" << GetName() << "," << this << ") DeleteWatch: object is being destroyed" << endl ;
+   cxcoutD(Tracing) << "RooAbsArg::dtor(" << GetName() << "," << this << ") DeleteWatch: object is being destroyed" << std::endl ;
    first = false ;
       }
 
       cxcoutD(Tracing)  << fName << "::" << ClassName() << ":~RooAbsArg: dependent \""
-             << client->GetName() << "\" should have been deleted first" << endl ;
+             << client->GetName() << "\" should have been deleted first" << std::endl ;
     }
   }
 
@@ -329,12 +328,12 @@ void RooAbsArg::addServer(RooAbsArg& server, bool valueProp, bool shapeProp, std
   if (_prohibitServerRedirect) {
     cxcoutF(LinkStateMgmt) << "RooAbsArg::addServer(" << this << "," << GetName()
             << "): PROHIBITED SERVER ADDITION REQUESTED: adding server " << server.GetName()
-            << "(" << &server << ") for " << (valueProp?"value ":"") << (shapeProp?"shape":"") << endl ;
+            << "(" << &server << ") for " << (valueProp?"value ":"") << (shapeProp?"shape":"") << std::endl ;
     throw std::logic_error("PROHIBITED SERVER ADDITION REQUESTED in RooAbsArg::addServer");
   }
 
   cxcoutD(LinkStateMgmt) << "RooAbsArg::addServer(" << this << "," << GetName() << "): adding server " << server.GetName()
-          << "(" << &server << ") for " << (valueProp?"value ":"") << (shapeProp?"shape":"") << endl ;
+          << "(" << &server << ") for " << (valueProp?"value ":"") << (shapeProp?"shape":"") << std::endl ;
 
   if (server.operMode()==ADirty && operMode()!=ADirty && valueProp) {
     setOperMode(ADirty) ;
@@ -387,7 +386,7 @@ void RooAbsArg::removeServer(RooAbsArg& server, bool force)
 
   if (_verboseDirty) {
     cxcoutD(LinkStateMgmt) << "RooAbsArg::removeServer(" << GetName() << "): removing server "
-            << server.GetName() << "(" << &server << ")" << endl ;
+            << server.GetName() << "(" << &server << ")" << std::endl ;
   }
 
   // Remove server link to given server
@@ -439,14 +438,14 @@ void RooAbsArg::changeServer(RooAbsArg& server, bool valueProp, bool shapeProp)
 {
   if (!_serverList.containsByNamePtr(&server)) {
     coutE(LinkStateMgmt) << "RooAbsArg::changeServer(" << GetName() << "): Server "
-    << server.GetName() << " not registered" << endl ;
+    << server.GetName() << " not registered" << std::endl ;
     return ;
   }
 
   // This condition should not happen, but check anyway
   if (!server._clientList.containsByNamePtr(this)) {
     coutE(LinkStateMgmt) << "RooAbsArg::changeServer(" << GetName() << "): Server "
-          << server.GetName() << " doesn't have us registered as client" << endl ;
+          << server.GetName() << " doesn't have us registered as client" << std::endl ;
     return ;
   }
 
@@ -499,7 +498,7 @@ void RooAbsArg::branchNodeServerList(RooAbsCollection* list, const RooAbsArg* ar
 void RooAbsArg::treeNodeServerList(RooAbsCollection* list, const RooAbsArg* arg, bool doBranch, bool doLeaf, bool valueOnly, bool recurseFundamental) const
 {
 //   if (arg==0) {
-//     cout << "treeNodeServerList(" << GetName() << ") doBranch=" << (doBranch?"T":"F") << " doLeaf = " << (doLeaf?"T":"F") << " valueOnly=" << (valueOnly?"T":"F") << endl ;
+//     std::cout << "treeNodeServerList(" << GetName() << ") doBranch=" << (doBranch?"T":"F") << " doLeaf = " << (doLeaf?"T":"F") << " valueOnly=" << (valueOnly?"T":"F") << std::endl ;
 //   }
 
   if (!arg) {
@@ -777,25 +776,6 @@ bool RooAbsArg::getObservables(const RooAbsCollection* dataList, RooArgSet& outp
 }
 
 
-/// \deprecated Use getObservables()
-RooFit::OwningPtr<RooArgSet> RooAbsArg::getDependents(const RooArgSet &set) const
-{
-   return getObservables(set);
-}
-
-/// \deprecated Use getObservables()
-RooFit::OwningPtr<RooArgSet> RooAbsArg::getDependents(const RooAbsData *set) const
-{
-   return getObservables(set);
-}
-
-/// \deprecated Use getObservables()
-RooFit::OwningPtr<RooArgSet> RooAbsArg::getDependents(const RooArgSet *depList) const
-{
-   return getObservables(depList);
-}
-
-
 ////////////////////////////////////////////////////////////////////////////////
 /// Create a RooArgSet with all components (branch nodes) of the
 /// expression tree headed by this object.
@@ -832,7 +812,7 @@ bool RooAbsArg::recursiveCheckObservables(const RooArgSet* nset) const
   for(RooAbsArg * arg : nodeList) {
     if (arg->getAttribute("ServerDied")) {
       coutE(LinkStateMgmt) << "RooAbsArg::recursiveCheckObservables(" << GetName() << "): ERROR: one or more servers of node "
-            << arg->GetName() << " no longer exists!" << endl ;
+            << arg->GetName() << " no longer exists!" << std::endl ;
       arg->Print("v") ;
       ret = true ;
     }
@@ -949,7 +929,7 @@ void RooAbsArg::setValueDirty(const RooAbsArg* source)
   } else if (source==this) {
     // Cyclical dependency, abort
     coutE(LinkStateMgmt) << "RooAbsArg::setValueDirty(" << GetName()
-          << "): cyclical dependency detected, source = " << source->GetName() << endl ;
+          << "): cyclical dependency detected, source = " << source->GetName() << std::endl ;
     //assert(0) ;
     return ;
   }
@@ -957,7 +937,7 @@ void RooAbsArg::setValueDirty(const RooAbsArg* source)
   // Propagate dirty flag to all clients if this is a down->up transition
   if (_verboseDirty) {
     cxcoutD(LinkStateMgmt) << "RooAbsArg::setValueDirty(" << (source?source->GetName():"self") << "->" << GetName() << "," << this
-            << "): dirty flag " << (_valueDirty?"already ":"") << "raised" << endl ;
+            << "): dirty flag " << (_valueDirty?"already ":"") << "raised" << std::endl ;
   }
 
   _valueDirty = true ;
@@ -979,7 +959,7 @@ void RooAbsArg::setShapeDirty(const RooAbsArg* source)
 {
   if (_verboseDirty) {
     cxcoutD(LinkStateMgmt) << "RooAbsArg::setShapeDirty(" << GetName()
-            << "): dirty flag " << (_shapeDirty?"already ":"") << "raised" << endl ;
+            << "): dirty flag " << (_shapeDirty?"already ":"") << "raised" << std::endl ;
   }
 
   if (_clientListShape.empty()) {
@@ -993,7 +973,7 @@ void RooAbsArg::setShapeDirty(const RooAbsArg* source)
   } else if (source==this) {
     // Cyclical dependency, abort
     coutE(LinkStateMgmt) << "RooAbsArg::setShapeDirty(" << GetName()
-    << "): cyclical dependency detected" << endl ;
+    << "): cyclical dependency detected" << std::endl ;
     return ;
   }
 
@@ -1012,8 +992,8 @@ void RooAbsArg::setShapeDirty(const RooAbsArg* source)
 ////////////////////////////////////////////////////////////////////////////////
 /// Replace all direct servers of this object with the new servers in `newServerList`.
 /// This substitutes objects that we receive values from with new objects that have the same name.
-/// \see recursiveRedirectServers() Use recursive version if servers that are only indirectly serving this object should be replaced as well.
-/// \see redirectServers() If only the direct servers of an object need to be replaced.
+/// See: recursiveRedirectServers() Use recursive version if servers that are only indirectly serving this object should be replaced as well.
+/// See: redirectServers() If only the direct servers of an object need to be replaced.
 ///
 /// Note that changing the types of objects is generally allowed, but can be wrong if the interface of an object changes.
 /// For example, one can reparametrise a model by substituting a variable with a function:
@@ -1102,7 +1082,7 @@ bool RooAbsArg::redirectServers(const RooAbsCollection& newSetOrig, bool mustRep
       auto ap = dynamic_cast<const RooArgProxy*>(p);
       coutE(LinkStateMgmt) << "RooAbsArg::redirectServers(" << GetName()
               << "): ERROR, proxy '" << p->name()
-              << "' with arg '" << (ap ? ap->absArg()->GetName() : "<could not cast>") << "' could not be adjusted" << endl;
+              << "' with arg '" << (ap ? ap->absArg()->GetName() : "<could not cast>") << "' could not be adjusted" << std::endl;
       ret = true ;
     }
   }
@@ -1260,7 +1240,7 @@ bool recursiveRedirectServersImpl(RooAbsArg *arg, RooAbsCollection const &newSet
    oocxcoutD(arg, LinkStateMgmt) << "RooAbsArg::recursiveRedirectServers(" << arg << "," << arg->GetName()
                                  << ") newSet = " << newSet << " mustReplaceAll = " << (mustReplaceAll ? "T" : "F")
                                  << " nameChange = " << (nameChange ? "T" : "F")
-                                 << " recurseInNewSet = " << (recurseInNewSet ? "T" : "F") << endl;
+                                 << " recurseInNewSet = " << (recurseInNewSet ? "T" : "F") << std::endl;
 
    // Do redirect on self (identify operation as recursion step)
    ret |= arg->redirectServers(newSet, mustReplaceAll, nameChange, true);
@@ -1304,7 +1284,7 @@ bool RooAbsArg::recursiveRedirectServers(RooAbsCollection const &newSet, bool mu
 /// overload this function, don't forget to also call the function of the
 /// base class.
 ///
-/// \see redirectServers() For a detailed explanation of the function parameters.
+/// See: redirectServers() For a detailed explanation of the function parameters.
 ///
 // \param[in] newServerList One of the original parameters passed to redirectServers().
 // \param[in] mustReplaceAll One of the original parameters passed to redirectServers().
@@ -1332,13 +1312,13 @@ void RooAbsArg::registerProxy(RooArgProxy& proxy)
   if (_proxyList.FindObject(&proxy)) {
     coutE(LinkStateMgmt) << "RooAbsArg::registerProxy(" << GetName() << "): proxy named "
           << proxy.GetName() << " for arg " << proxy.absArg()->GetName()
-          << " already registered" << endl ;
+          << " already registered" << std::endl ;
     return ;
   }
 
-//   cout << (void*)this << " " << GetName() << ": registering proxy "
+//   std::cout << (void*)this << " " << GetName() << ": registering proxy "
 //        << (void*)&proxy << " with name " << proxy.name() << " in mode "
-//        << (proxy.isValueServer()?"V":"-") << (proxy.isShapeServer()?"S":"-") << endl ;
+//        << (proxy.isValueServer()?"V":"-") << (proxy.isShapeServer()?"S":"-") << std::endl ;
 
   // Register proxied object as server
   if (proxy.absArg()) {
@@ -1375,7 +1355,7 @@ void RooAbsArg::registerProxy(RooSetProxy& proxy)
   // Every proxy can be registered only once
   if (_proxyList.FindObject(&proxy)) {
     coutE(LinkStateMgmt) << "RooAbsArg::registerProxy(" << GetName() << "): proxy named "
-          << proxy.GetName() << " already registered" << endl ;
+          << proxy.GetName() << " already registered" << std::endl ;
     return ;
   }
 
@@ -1410,7 +1390,7 @@ void RooAbsArg::registerProxy(RooListProxy& proxy)
   // Every proxy can be registered only once
   if (_proxyList.FindObject(&proxy)) {
     coutE(LinkStateMgmt) << "RooAbsArg::registerProxy(" << GetName() << "): proxy named "
-          << proxy.GetName() << " already registered" << endl ;
+          << proxy.GetName() << " already registered" << std::endl ;
     return ;
   }
 
@@ -1419,7 +1399,7 @@ void RooAbsArg::registerProxy(RooListProxy& proxy)
   _proxyList.Add(&proxy) ;
   _proxyListCache.isDirty = true;
   if (_proxyList.GetEntries()!=nProxyOld+1) {
-    cout << "RooAbsArg::registerProxy(" << GetName() << ") proxy registration failure! nold=" << nProxyOld << " nnew=" << _proxyList.GetEntries() << endl ;
+    std::cout << "RooAbsArg::registerProxy(" << GetName() << ") proxy registration failure! nold=" << nProxyOld << " nnew=" << _proxyList.GetEntries() << std::endl ;
   }
 }
 
@@ -1497,7 +1477,7 @@ void RooAbsArg::setProxyNormSet(const RooArgSet* nset)
 void RooAbsArg::attachToTree(TTree& ,Int_t)
 {
   coutE(Contents) << "RooAbsArg::attachToTree(" << GetName()
-        << "): Cannot be attached to a TTree" << endl ;
+        << "): Cannot be attached to a TTree" << std::endl ;
 }
 
 
@@ -1588,7 +1568,7 @@ Int_t RooAbsArg::defaultPrintContents(Option_t* /*opt*/) const
 
 void RooAbsArg::printMultiline(ostream& os, Int_t /*contents*/, bool /*verbose*/, TString indent) const
 {
-  os << indent << "--- RooAbsArg ---" << endl;
+  os << indent << "--- RooAbsArg ---" << std::endl;
   // dirty state flags
   os << indent << "  Value State: " ;
   switch(_operMode) {
@@ -1596,16 +1576,16 @@ void RooAbsArg::printMultiline(ostream& os, Int_t /*contents*/, bool /*verbose*/
   case AClean: os << "FORCED clean" ; break ;
   case Auto: os << (isValueDirty() ? "DIRTY":"clean") ; break ;
   }
-  os << endl
-     << indent << "  Shape State: " << (isShapeDirty() ? "DIRTY":"clean") << endl;
+  os << std::endl
+     << indent << "  Shape State: " << (isShapeDirty() ? "DIRTY":"clean") << std::endl;
   // attribute list
   os << indent << "  Attributes: " ;
   printAttribList(os) ;
-  os << endl ;
+  os << std::endl ;
   // our memory address (for x-referencing with client addresses of other args)
-  os << indent << "  Address: " << (void*)this << endl;
+  os << indent << "  Address: " << (void*)this << std::endl;
   // client list
-  os << indent << "  Clients: " << endl;
+  os << indent << "  Clients: " << std::endl;
   for (const auto client : _clientList) {
     os << indent << "    (" << (void*)client  << ","
        << (_clientListValue.containsByNamePtr(client)?"V":"-")
@@ -1615,7 +1595,7 @@ void RooAbsArg::printMultiline(ostream& os, Int_t /*contents*/, bool /*verbose*/
   }
 
   // server list
-  os << indent << "  Servers: " << endl;
+  os << indent << "  Servers: " << std::endl;
   for (const auto server : _serverList) {
     os << indent << "    (" << (void*)server << ","
        << (server->_clientListValue.containsByNamePtr(this)?"V":"-")
@@ -1760,13 +1740,13 @@ void RooAbsArg::printDirty(bool depth) const
     }
 
   } else {
-    cout << GetName() << " : " ;
+    std::cout << GetName() << " : " ;
     switch (_operMode) {
-    case AClean: cout << "FORCED clean" ; break ;
-    case ADirty: cout << "FORCED DIRTY" ; break ;
-    case Auto:   cout << "Auto  " << (isValueDirty()?"DIRTY":"clean") ;
+    case AClean: std::cout << "FORCED clean" ; break ;
+    case ADirty: std::cout << "FORCED DIRTY" ; break ;
+    case Auto:   std::cout << "Auto  " << (isValueDirty()?"DIRTY":"clean") ;
     }
-    cout << endl ;
+    std::cout << std::endl ;
   }
 }
 
@@ -1786,7 +1766,7 @@ void RooAbsArg::optimizeCacheMode(const RooArgSet& observables)
   optimizeCacheMode(observables,opt,proc) ;
 
   coutI(Optimization) << "RooAbsArg::optimizeCacheMode(" << GetName() << ") nodes " << opt << " depend on observables, "
-         << "changing cache operation mode from change tracking to unconditional evaluation" << endl ;
+         << "changing cache operation mode from change tracking to unconditional evaluation" << std::endl ;
 }
 
 
@@ -1808,7 +1788,7 @@ void RooAbsArg::optimizeCacheMode(const RooArgSet& observables, RooArgSet& optim
 
   // Terminate call if this node was already processed (tree structure may be cyclical)
   // LM : RooLinkedList::findArg looks by name and not but by object pointer,
-  //  should one use RooLinkedList::FindObject (look byt pointer) instead of findArg when
+  //  should one use RooLinkedList::FindObject (look by pointer) instead of findArg when
   // tree contains nodes with the same name ?
   // Add an info message if the require node does not exist but a different node already exists with same name
 
@@ -1822,7 +1802,7 @@ void RooAbsArg::optimizeCacheMode(const RooArgSet& observables, RooArgSet& optim
   if (obj) {
      // here for nodes with duplicate names
      cxcoutI(Optimization) << "RooAbsArg::optimizeCacheMode(" << GetName() << " node " << this << " exists already as "
-                           << obj << " but with the SAME name !" << endl;
+                           << obj << " but with the SAME name !" << std::endl;
   }
 
   processedNodes.Add(this);
@@ -1831,7 +1811,7 @@ void RooAbsArg::optimizeCacheMode(const RooArgSet& observables, RooArgSet& optim
   if (dependsOnValue(observables)) {
 
     if (dynamic_cast<RooRealIntegral*>(this)) {
-      cxcoutI(Integration) << "RooAbsArg::optimizeCacheMode(" << GetName() << ") integral depends on value of one or more observables and will be evaluated for every event" << endl ;
+      cxcoutI(Integration) << "RooAbsArg::optimizeCacheMode(" << GetName() << ") integral depends on value of one or more observables and will be evaluated for every event" << std::endl ;
     }
     optimizedNodes.add(*this,true) ;
     if (operMode()==AClean) {
@@ -1863,7 +1843,7 @@ bool RooAbsArg::findConstantNodes(const RooArgSet& observables, RooArgSet& cache
 
   // If node can be optimized and hasn't been identified yet, add it to the list
   coutI(Optimization) << "RooAbsArg::findConstantNodes(" << GetName() << "): components "
-         << cacheList << " depend exclusively on constant parameters and will be precalculated and cached" << endl ;
+         << cacheList << " depend exclusively on constant parameters and will be precalculated and cached" << std::endl ;
 
   return ret ;
 }
@@ -1914,7 +1894,7 @@ bool RooAbsArg::findConstantNodes(const RooArgSet& observables, RooArgSet& cache
     if (!cacheList.find(*this) && dependsOnValue(observables) && !observables.find(*this) ) {
 
       // Add to cache list
-      cxcoutD(Optimization) << "RooAbsArg::findConstantNodes(" << GetName() << ") adding self to list of constant nodes" << endl ;
+      cxcoutD(Optimization) << "RooAbsArg::findConstantNodes(" << GetName() << ") adding self to list of constant nodes" << std::endl ;
 
       if (canOpt) setAttribute("ConstantExpressionCached") ;
       cacheList.add(*this,false) ;
@@ -1993,7 +1973,7 @@ void RooAbsArg::printCompactTree(const char* indent, const char* filename, const
     ofstream ofs(filename) ;
     printCompactTree(ofs,indent,namePat,client) ;
   } else {
-    printCompactTree(cout,indent,namePat,client) ;
+    printCompactTree(std::cout,indent,namePat,client) ;
   }
 }
 
@@ -2025,7 +2005,7 @@ void RooAbsArg::printCompactTree(ostream& os, const char* indent, const char* na
       case ADirty: os << " [ADIRTY] " ; break ;
       }
     }
-    os << endl ;
+    os << std::endl ;
 
     for (Int_t i=0 ;i<numCaches() ; i++) {
       getCache(i)->printCompactTreeHook(os,indent) ;
@@ -2056,7 +2036,7 @@ void RooAbsArg::printComponentTree(const char* indent, const char* namePat, Int_
   if (InheritsFrom("RooConstVar")) return ;
 
   if ( !namePat || TString(GetName()).Contains(namePat)) {
-    cout << indent ;
+    std::cout << indent ;
     Print() ;
   }
 
@@ -2170,7 +2150,7 @@ void RooAbsArg::graphVizTree(const char* fileName, const char* delimiter, bool u
 {
   ofstream ofs(fileName) ;
   if (!ofs) {
-    coutE(InputArguments) << "RooAbsArg::graphVizTree() ERROR: Cannot open graphViz output file with name " << fileName << endl ;
+    coutE(InputArguments) << "RooAbsArg::graphVizTree() ERROR: Cannot open graphViz output file with name " << fileName << std::endl ;
     return ;
   }
   graphVizTree(ofs, delimiter, useTitle, useLatex) ;
@@ -2187,14 +2167,14 @@ void RooAbsArg::graphVizTree(const char* fileName, const char* delimiter, bool u
 void RooAbsArg::graphVizTree(ostream& os, const char* delimiter, bool useTitle, bool useLatex)
 {
   if (!os) {
-    coutE(InputArguments) << "RooAbsArg::graphVizTree() ERROR: output stream provided as input argument is in invalid state" << endl ;
+    coutE(InputArguments) << "RooAbsArg::graphVizTree() ERROR: output stream provided as input argument is in invalid state" << std::endl ;
   }
 
   // silent warning messages coming when evaluating a RooAddPdf without a normalization set
   RooHelpers::LocalChangeMsgLevel locmsg(RooFit::WARNING, 0u, RooFit::Eval, false);
 
   // Write header
-  os << "digraph \"" << GetName() << "\"{" << endl ;
+  os << "digraph \"" << GetName() << "\"{" << std::endl ;
 
   // First list all the tree nodes
   RooArgSet nodeSet ;
@@ -2220,7 +2200,7 @@ void RooAbsArg::graphVizTree(ostream& os, const char* delimiter, bool useTitle,
     }
 
     os << "\"" << nodeName << "\" [ color=" << (node->isFundamental()?"blue":"red")
-       << ", label=\"" << nodeType << delimiter << nodeLabel << "\"];" << endl ;
+       << ", label=\"" << nodeType << delimiter << nodeLabel << "\"];" << std::endl ;
 
   }
 
@@ -2230,11 +2210,11 @@ void RooAbsArg::graphVizTree(ostream& os, const char* delimiter, bool useTitle,
 
   // And write them out
   for(auto const& link : links) {
-    os << "\"" << link.first->GetName() << "\" -> \"" << link.second->GetName() << "\";" << endl ;
+    os << "\"" << link.first->GetName() << "\" -> \"" << link.second->GetName() << "\";" << std::endl ;
   }
 
   // Write trailer
-  os << "}" << endl ;
+  os << "}" << std::endl ;
 
 }
 
@@ -2266,7 +2246,7 @@ bool RooAbsArg::addOwnedComponents(const RooAbsCollection& comps)
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Take ownership of the contents of 'comps'. Different from the overload that
-/// taked the RooArgSet by `const&`, this version can also take an owning
+/// takes the RooArgSet by `const&`, this version can also take an owning
 /// RooArgSet without error, because the ownership will not be ambiguous afterwards.
 
 bool RooAbsArg::addOwnedComponents(RooAbsCollection&& comps)
@@ -2311,7 +2291,7 @@ RooAbsArg* RooAbsArg::cloneTree(const char* newname) const
 void RooAbsArg::attachToStore(RooAbsDataStore& store)
 {
   if (dynamic_cast<RooTreeDataStore*>(&store)) {
-    attachToTree((static_cast<RooTreeDataStore&>(store)).tree()) ;
+    attachToTree(*static_cast<RooTreeDataStore&>(store).tree()) ;
   } else if (dynamic_cast<RooVectorDataStore*>(&store)) {
     attachToVStore(static_cast<RooVectorDataStore&>(store)) ;
   }
@@ -2369,7 +2349,7 @@ void RooAbsArg::SetName(const char* name)
   TNamed::SetName(name) ;
   auto newPtr = RooNameReg::instance().constPtr(GetName()) ;
   if (newPtr != _namePtr) {
-    //cout << "Rename '" << _namePtr->GetName() << "' to '" << name << "' (set flag in new name)" << endl;
+    //cout << "Rename '" << _namePtr->GetName() << "' to '" << name << "' (set flag in new name)" << std::endl;
     _namePtr = newPtr;
     const_cast<TNamed*>(_namePtr)->SetBit(RooNameReg::kRenamedArg);
     RooNameReg::incrementRenameCounter();
@@ -2534,23 +2514,6 @@ std::unique_ptr<RooAbsArg> RooAbsArg::compileForNormSet(RooArgSet const & normSe
 }
 
 
-////////////////////////////////////////////////////////////////////////////////
-/// This function defines a translation for each RooAbsReal based object that can be used
-/// to express the class as simple C++ code. The function adds the code represented by
-/// each class as an std::string (that is later concatenated with code strings from translate calls)
-/// to form the C++ code that AD tools can understand. Any class that wants to support AD, has to
-/// implement this function.
-///
-/// \param[in] ctx An object to manage auxiliary information for code-squashing. Also takes the
-/// code string that this class outputs into the squashed code through the 'addToCodeBody' function.
-void RooAbsArg::translate(RooFit::Detail::CodeSquashContext & /*ctx*/) const
-{
-   std::stringstream errorMsg;
-   errorMsg << "Translate function for class \"" << ClassName() << "\" has not yet been implemented.";
-   coutE(Minimization) << errorMsg.str() << std::endl;
-   throw std::runtime_error(errorMsg.str().c_str());
-}
-
 /// Sets the token for retrieving results in the BatchMode. For internal use only.
 void RooAbsArg::setDataToken(std::size_t index)
 {
