@@ -97,9 +97,8 @@ void RooSimultaneous::InitializationOutput::addPdf(const RooAbsPdf &pdf, std::st
    finalCatLabels.emplace_back(catLabel);
 }
 
-using std::string, std::endl;
+using std::string;
 
-ClassImp(RooSimultaneous);
 
 
 
@@ -273,7 +272,7 @@ RooSimultaneous::initialize(std::string const& name, RooAbsCategoryLValue &inInd
         string superLabel = superIndex->getCurrentLabel() ;
         out->addPdf(*citem.second.pdf,superLabel);
         oocxcoutD(static_cast<RooAbsArg*>(nullptr), InputArguments) << msgPrefix
-                << "assigning pdf " << citem.second.pdf->GetName() << " to super label " << superLabel << endl ;
+                << "assigning pdf " << citem.second.pdf->GetName() << " to super label " << superLabel << std::endl ;
       }
     } else {
 
@@ -291,11 +290,11 @@ RooSimultaneous::initialize(std::string const& name, RooAbsCategoryLValue &inInd
             out->addPdf(*compPdf,superLabel);
             oocxcoutD(static_cast<RooAbsArg*>(nullptr), InputArguments) << msgPrefix
                     << "assigning pdf " << compPdf->GetName() << "(member of " << citem.second.pdf->GetName()
-                    << ") to super label " << superLabel << endl ;
+                    << ") to super label " << superLabel << std::endl ;
           } else {
             oocoutW(nullptr, InputArguments) << msgPrefix << "WARNING: No p.d.f. associated with label "
                 << type.second << " for component RooSimultaneous p.d.f " << citem.second.pdf->GetName()
-                << "which is associated with master index label " << citem.first << endl ;
+                << "which is associated with master index label " << citem.first << std::endl ;
           }
         }
 
@@ -317,11 +316,11 @@ RooSimultaneous::initialize(std::string const& name, RooAbsCategoryLValue &inInd
               out->addPdf(*compPdf,superLabel);
               oocxcoutD(static_cast<RooAbsArg*>(nullptr), InputArguments) << msgPrefix
                       << "assigning pdf " << compPdf->GetName() << "(member of " << citem.second.pdf->GetName()
-                      << ") to super label " << superLabel << endl ;
+                      << ") to super label " << superLabel << std::endl ;
             } else {
               oocoutW(nullptr, InputArguments) << msgPrefix << "WARNING: No p.d.f. associated with label "
                   << stype.second << " for component RooSimultaneous p.d.f " << citem.second.pdf->GetName()
-                  << "which is associated with master index label " << citem.first << endl ;
+                  << "which is associated with master index label " << citem.first << std::endl ;
             }
           }
         }
@@ -392,14 +391,14 @@ bool RooSimultaneous::addPdf(const RooAbsPdf& pdf, const char* catLabel)
   // PDFs cannot overlap with the index category
   if (pdf.dependsOn(_indexCat.arg())) {
     coutE(InputArguments) << "RooSimultaneous::addPdf(" << GetName() << "): PDF '" << pdf.GetName()
-           << "' overlaps with index category '" << _indexCat.arg().GetName() << "'."<< endl ;
+           << "' overlaps with index category '" << _indexCat.arg().GetName() << "'."<< std::endl ;
     return true ;
   }
 
   // Each index state can only have one PDF associated with it
   if (_pdfProxyList.FindObject(catLabel)) {
     coutE(InputArguments) << "RooSimultaneous::addPdf(" << GetName() << "): index state '"
-           << catLabel << "' has already an associated PDF." << endl ;
+           << catLabel << "' has already an associated PDF." << std::endl ;
     return true ;
   }
 
@@ -408,7 +407,7 @@ bool RooSimultaneous::addPdf(const RooAbsPdf& pdf, const char* catLabel)
 
     coutE(InputArguments) << "RooSimultaneous::addPdf(" << GetName()
            << ") ERROR: you cannot add a RooSimultaneous component to a RooSimultaneous using addPdf()."
-           << " Use the constructor with RooArgList if input p.d.f.s or the map<string,RooAbsPdf&> instead." << endl ;
+           << " Use the constructor with RooArgList if input p.d.f.s or the map<string,RooAbsPdf&> instead." << std::endl ;
     return true ;
 
   } else {
@@ -608,7 +607,7 @@ RooPlot* RooSimultaneous::plotOn(RooPlot *frame, RooLinkedList& cmdList) const
     return frame ;
   }
 
-  const RooAbsData* projData = static_cast<const RooAbsData*>(pc.getObject("projData")) ;
+  RooAbsData* projData = static_cast<RooAbsData*>(pc.getObject("projData")) ;
   const RooArgSet* projDataSet = pc.getSet("projDataSet");
   const RooArgSet* sliceSetTmp = pc.getSet("sliceSet") ;
   std::unique_ptr<RooArgSet> sliceSet( sliceSetTmp ? (static_cast<RooArgSet*>(sliceSetTmp->Clone())) : nullptr );
@@ -646,7 +645,7 @@ RooPlot* RooSimultaneous::plotOn(RooPlot *frame, RooLinkedList& cmdList) const
 
   // Check if we have a projection dataset
   if (!projData) {
-    coutE(InputArguments) << "RooSimultaneous::plotOn(" << GetName() << ") ERROR: must have a projection dataset for index category" << endl ;
+    coutE(InputArguments) << "RooSimultaneous::plotOn(" << GetName() << ") ERROR: must have a projection dataset for index category" << std::endl ;
     return frame ;
   }
 
@@ -662,7 +661,7 @@ RooPlot* RooSimultaneous::plotOn(RooPlot *frame, RooLinkedList& cmdList) const
         projectedVars.remove(*arg) ;
       } else {
         coutI(Plotting) << "RooAbsReal::plotOn(" << GetName() << ") slice variable "
-            << sliceArg->GetName() << " was not projected anyway" << endl ;
+            << sliceArg->GetName() << " was not projected anyway" << std::endl ;
       }
     }
   } else if (projSet) {
@@ -675,12 +674,12 @@ RooPlot* RooSimultaneous::plotOn(RooPlot *frame, RooLinkedList& cmdList) const
 
   if (!_indexCat.arg().isDerived()) {
     // *** Error checking for a fundamental index category ***
-    //cout << "RooSim::plotOn: index is fundamental" << endl ;
+    //cout << "RooSim::plotOn: index is fundamental" << std::endl ;
 
     // Check that the provided projection dataset contains our index variable
     if (!projData->get()->find(_indexCat.arg().GetName())) {
       coutE(Plotting) << "RooSimultaneous::plotOn(" << GetName() << ") ERROR: Projection over index category "
-            << "requested, but projection data set doesn't contain index category" << endl ;
+            << "requested, but projection data set doesn't contain index category" << std::endl ;
       return frame ;
     }
 
@@ -717,7 +716,7 @@ RooPlot* RooSimultaneous::plotOn(RooPlot *frame, RooLinkedList& cmdList) const
     if (!allServers) {
       coutE(Plotting) << "RooSimultaneous::plotOn(" << GetName()
           << ") ERROR: Projection dataset doesn't contain complete set of index categories to do projection."
-          << "\n\tcategory " << missing << " is missing." << endl ;
+          << "\n\tcategory " << missing << " is missing." << std::endl ;
       return frame ;
     }
 
@@ -743,35 +742,27 @@ RooPlot* RooSimultaneous::plotOn(RooPlot *frame, RooLinkedList& cmdList) const
   if (!projIndex) {
 
     coutI(Plotting) << "RooSimultaneous::plotOn(" << GetName() << ") plot on " << frame->getPlotVar()->GetName()
-          << " represents a slice in the index category ("  << _indexCat.arg().GetName() << ")" << endl ;
+          << " represents a slice in the index category ("  << _indexCat.arg().GetName() << ")" << std::endl ;
 
     // Reduce projData: take out fitCat (component) columns and entries that don't match selected slice
     // Construct cut string to only select projection data event that match the current slice
 
     // Make cut string to exclude rows from projection data
-    TString cutString ;
-    bool first(true) ;
-    for (const auto arg : *idxCompSliceSet) {
-      auto idxComp = static_cast<RooCategory*>(arg);
-      RooAbsArg* slicedComponent = nullptr;
-      if (sliceSet && (slicedComponent = sliceSet->find(*idxComp)) != nullptr) {
-        auto theCat = static_cast<const RooAbsCategory*>(slicedComponent);
-        idxComp->setIndex(theCat->getCurrentIndex(), false);
+    if (sliceSet) {
+      for (auto *idxComp : static_range_cast<RooCategory *>(*idxCompSliceSet)) {
+        if (auto* slicedComponent = static_cast<const RooAbsCategory*>(sliceSet->find(*idxComp))) {
+          idxComp->setIndex(slicedComponent->getCurrentIndex(), false);
+        }
       }
-
-      if (!first) {
-        cutString.Append("&&") ;
-      } else {
-        first=false ;
-      }
-      cutString.Append(Form("%s==%d",idxComp->GetName(),idxComp->getCurrentIndex())) ;
     }
+    std::string cutString = RooFit::Detail::makeSliceCutString(*idxCompSliceSet);
 
     // Make temporary projData without RooSim index category components
     RooArgSet projDataVars(*projData->get()) ;
     projDataVars.remove(*idxCompSliceSet,true,true) ;
 
-    std::unique_ptr<RooAbsData> projDataTmp( const_cast<RooAbsData*>(projData)->reduce(projDataVars,cutString) );
+    std::unique_ptr<RooAbsData>
+       projDataTmp(projData->reduce(RooFit::SelectVars(projDataVars), RooFit::Cut(cutString.c_str())));
 
     // Override normalization and projection dataset
     RooCmdArg tmp1 = RooFit::Normalization(scaleFactor*wTable->getFrac(idxCatClone->getCurrentLabel()),stype) ;
@@ -837,19 +828,7 @@ RooPlot* RooSimultaneous::plotOn(RooPlot *frame, RooLinkedList& cmdList) const
   if (projData) {
 
     // Construct cut string to only select projection data event that match the current slice
-    TString cutString ;
-    if (!idxCompSliceSet->empty()) {
-      bool first(true) ;
-      for (const auto idxSliceCompArg : *idxCompSliceSet) {
-        const auto idxSliceComp = static_cast<RooAbsCategory*>(idxSliceCompArg);
-        if (!first) {
-          cutString.Append("&&") ;
-        } else {
-          first=false ;
-        }
-        cutString.Append(Form("%s==%d",idxSliceComp->GetName(),idxSliceComp->getCurrentIndex())) ;
-      }
-    }
+    std::string cutString = RooFit::Detail::makeSliceCutString(*idxCompSliceSet);
 
     // Make temporary projData without RooSim index category components
     RooArgSet projDataVars(*projData->get()) ;
@@ -858,11 +837,7 @@ RooPlot* RooSimultaneous::plotOn(RooPlot *frame, RooLinkedList& cmdList) const
 
     projDataVars.remove(idxCatServers,true,true) ;
 
-    if (!idxCompSliceSet->empty()) {
-      projDataTmp = std::unique_ptr<RooAbsData>{const_cast<RooAbsData*>(projData)->reduce(projDataVars,cutString)};
-    } else {
-      projDataTmp = std::unique_ptr<RooAbsData>{const_cast<RooAbsData*>(projData)->reduce(projDataVars)};
-    }
+    projDataTmp = std::unique_ptr<RooAbsData>{projData->reduce(RooFit::SelectVars(projDataVars), RooFit::Cut(cutString.c_str()))};
 
 
 
@@ -875,18 +850,18 @@ RooPlot* RooSimultaneous::plotOn(RooPlot *frame, RooLinkedList& cmdList) const
 
   if (_indexCat.arg().isDerived() && !idxCompSliceSet->empty()) {
     coutI(Plotting) << "RooSimultaneous::plotOn(" << GetName() << ") plot on " << frame->getPlotVar()->GetName()
-          << " represents a slice in index category components " << *idxCompSliceSet << endl ;
+          << " represents a slice in index category components " << *idxCompSliceSet << std::endl ;
 
     RooArgSet idxCompProjSet;
     _indexCat.arg().getObservables(frame->getNormVars(), idxCompProjSet) ;
     idxCompProjSet.remove(*idxCompSliceSet,true,true) ;
     if (!idxCompProjSet.empty()) {
       coutI(Plotting) << "RooSimultaneous::plotOn(" << GetName() << ") plot on " << frame->getPlotVar()->GetName()
-            << " averages with data index category components " << idxCompProjSet << endl ;
+            << " averages with data index category components " << idxCompProjSet << std::endl ;
     }
   } else {
     coutI(Plotting) << "RooSimultaneous::plotOn(" << GetName() << ") plot on " << frame->getPlotVar()->GetName()
-          << " averages with data index category (" << _indexCat.arg().GetName() << ")" << endl ;
+          << " averages with data index category (" << _indexCat.arg().GetName() << ")" << std::endl ;
   }
 
 
@@ -976,16 +951,24 @@ RooAbsGenContext* RooSimultaneous::genContext(const RooArgSet &vars, const RooDa
   RooArgSet catsAmongAllVars;
   allVars.selectCommon(flattenedCatList(), catsAmongAllVars);
 
-  // Not generating index cat: return context for pdf associated with present index state
+  // Not generating index cat: we better error out because it's not clear what
+  // the user expects here. Does the user want to generate according to the
+  // currently-selected pdf? Or does the user want to generate global
+  // observable values according to the union of all category pdfs?
+  // Print an error and tell the user what to do to explicitly.
   if(catsAmongAllVars.empty()) {
-    auto* proxy = static_cast<RooRealProxy*>(_pdfProxyList.FindObject(_indexCat->getCurrentLabel()));
-    if (!proxy) {
-      coutE(InputArguments) << "RooSimultaneous::genContext(" << GetName()
-             << ") ERROR: no PDF associated with current state ("
-             << _indexCat.arg().GetName() << "=" << _indexCat.arg().getCurrentLabel() << ")" << endl ;
-      return nullptr;
-    }
-    return static_cast<RooAbsPdf*>(proxy->absArg())->genContext(vars,prototype,auxProto,verbose) ;
+    coutE(InputArguments) << "RooSimultaneous::generateSimGlobal(" << GetName()
+           << ") asking to generate without the index category!\n"
+           << "It's not clear what to do. you probably want to either:\n"
+           << "\n"
+           << "    1. Generate according to the currently-selected pdf.\n"
+           << "       Please do this explicitly with:\n"
+           << "           simpdf->getPdf(simpdf->indexCat().getCurrentLabel())->generate(vars, ...)\n"
+           << "\n"
+           << "    1. Generate global observable values according to the union of all component pdfs.\n"
+           << "       For this, please use simpdf->generateSimGlobal(vars, ...)\n"
+           << std::endl;
+    return nullptr;
   }
 
   RooArgSet catsAmongProtoVars;
@@ -1032,10 +1015,30 @@ RooDataHist* RooSimultaneous::fillDataHist(RooDataHist *hist,
 
 
 ////////////////////////////////////////////////////////////////////////////////
-/// Special generator interface for generation of 'global observables' -- for RooStats tools
+/// Special generator interface for generation of 'global observables' -- for RooStats tools.
+///
+/// \note Why one can't just use RooAbsPdf::generate()? That's because when
+/// using the regular generate() method, a specific component pdf is selected
+/// for each generated entry according to the index category value. However,
+/// global observable values are independent of the current index category,
+/// which can best be illustrated with the case where a global observable
+/// corresponds to a nuisance parameter that is relevant for multiple channels.
+/// So the interpretation of what is an entry in the generated dataset is very
+/// different, hence the separate function.
 
 RooFit::OwningPtr<RooDataSet> RooSimultaneous::generateSimGlobal(const RooArgSet& whatVars, Int_t nEvents)
 {
+  // Generating the index category together with the global observables doesn't make any sense.
+  RooArgSet catsAmongAllVars;
+  whatVars.selectCommon(flattenedCatList(), catsAmongAllVars);
+  if(!catsAmongAllVars.empty()) {
+    coutE(InputArguments) << "RooSimultaneous::generateSimGlobal(" << GetName()
+           << ") asking to generate global obserables at the same time as the index category!\n"
+           << "This doesn't make any sense: global observables are generally not related to a specific channel.\n"
+           << std::endl;
+    return nullptr;
+  }
+
   // Make set with clone of variables (placeholder for output)
   RooArgSet globClone;
   whatVars.snapshot(globClone);
